@@ -1,33 +1,10 @@
-# Copyright (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#
-#  * Redistributions of source code must retain the above copyright notice,
-#    this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright notice,
-#    this list of conditions and the following disclaimer in the documentation
-#    and/or other materials provided with the distribution.
-#  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
-#    be used to endorse or promote products derived from this software without
-#    specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-
 """
-SoftLayer API bindings
+    SoftLayer.API
+    ~~~~~~~~~~~~~
+    SoftLayer API bindings
 
-See U{http://sldn.softlayer.com/article/Python}
+    :copyright: (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
+    :license: BSD, see LICENSE for more details.
 """
 from SoftLayer.consts import API_PUBLIC_ENDPOINT, API_PRIVATE_ENDPOINT, \
     USER_AGENT
@@ -57,11 +34,11 @@ class Client(object):
     :param integer timeout: timeout for API requests
     :param boolean verbose: prints details about every HTTP request if true
 
-    Usage::
+    Usage:
 
         >>> import SoftLayer
         >>> client = SoftLayer.Client(username="username", api_key="api_key")
-        >>> resp = client['SoftLayer_Account'].getObject()
+        >>> resp = client['Account'].getObject()
         >>> resp['companyName']
         'Your Company'
 
@@ -89,21 +66,23 @@ class Client(object):
 
     def add_raw_header(self, name, value):
         """ Set HTTP headers for API calls.
-        ..  deprecated:: 2.0.0
 
         :param name: the header name
         :param value: the header value
 
+        ..  deprecated:: 2.0.0
+
         """
         self._raw_headers[name] = value
 
     def add_header(self, name, value):
         """ Set a SoftLayer API call header.
-        ..  deprecated:: 2.0.0
 
         :param name: the header name
         :param value: the header value
 
+        ..  deprecated:: 2.0.0
+
         """
         name = name.strip()
         if name is None or name == '':
@@ -113,27 +92,29 @@ class Client(object):
 
     def remove_header(self, name):
         """ Remove a SoftLayer API call header.
-        ..  deprecated:: 2.0.0
 
         :param name: the header name
 
+        ..  deprecated:: 2.0.0
+
         """
         if name in self._headers:
             del self._headers[name.strip()]
 
     def set_authentication(self, username, api_key):
         """ Set user and key to authenticate a SoftLayer API call.
-        ..  deprecated:: 2.0.0
 
         Use this method if you wish to bypass the API_USER and API_KEY class
         constants and set custom authentication per API call.
 
-        See U{https://manage.softlayer.com/Administrative/apiKeychain} for more
+        See https://manage.softlayer.com/Administrative/apiKeychain for more
         information.
 
         :param username: the username to authenticate with
         :param api_key: the user's API key
 
+        ..  deprecated:: 2.0.0
+
         """
         self.add_header('authenticate', {
             'username': username.strip(),
@@ -142,7 +123,6 @@ class Client(object):
 
     def set_init_parameter(self, id):
         """ Set an initialization parameter header.
-        ..  deprecated:: 2.0.0
 
         Initialization parameters instantiate a SoftLayer API service object to
         act upon during your API method call. For instance, if your account has
@@ -150,11 +130,13 @@ class Client(object):
         of 1234 in the SoftLayer_Hardware_Server Service instructs the API to
         act on server record 1234 in your method calls.
 
-        See U{http://sldn.softlayer.com/article/Using-Initialization-Parameters-SoftLayer-API}  # NOQA
+        See http://sldn.softlayer.com/article/Using-Initialization-Parameters-SoftLayer-API  # NOQA
         for more information.
 
         :param id: the ID of the SoftLayer API object to instantiate
 
+        ..  deprecated:: 2.0.0
+
         """
         self.add_header(self._service_name + 'InitParameters', {
             'id': int(id)
@@ -162,17 +144,18 @@ class Client(object):
 
     def set_object_mask(self, mask):
         """ Set an object mask to a SoftLayer API call.
-        ..  deprecated:: 2.0.0
 
         Use an object mask to retrieve data related your API call's result.
         Object masks are skeleton objects, or strings that define nested
         relational properties to retrieve along with an object's local
         properties. See
-        U{http://sldn.softlayer.com/article/Using-Object-Masks-SoftLayer-API}
+        http://sldn.softlayer.com/article/Using-Object-Masks-SoftLayer-API
         for more information.
 
         :param mask: the object mask you wish to define
 
+        ..  deprecated:: 2.0.0
+
         """
         header = 'SoftLayer_ObjectMask'
 
@@ -183,7 +166,6 @@ class Client(object):
 
     def set_result_limit(self, limit, offset=0):
         """ Set a result limit on a SoftLayer API call.
-        ..  deprecated:: 2.0.0
 
         Many SoftLayer API methods return a group of results. These methods
         support a way to limit the number of results retrieved from the
@@ -191,7 +173,9 @@ class Client(object):
 
         :param limit: the number of results to limit a SoftLayer API call to
         :param offset: An optional offset at which to begin a SoftLayer API
-        call's returned result
+                       call's returned result
+
+        ..  deprecated:: 2.0.0
 
         """
         self.add_header('resultLimit', {
@@ -202,22 +186,37 @@ class Client(object):
     def __getitem__(self, name):
         """ Get a SoftLayer Service.
 
-        :param name: The name of the service. E.G. SoftLayer_Account
+        :param name: The name of the service. E.G. Account
+
+        Usage:
+            >>> client['Account']
+            <Service: Account>
 
         """
-        if not name.startswith(self._prefix):
-            name = self._prefix + name
         return Service(self, name)
 
-    def __call__(self, service, method, *args, **kwargs):
-        """ Perform a SoftLayer API call.
+    def call(self, service, method, *args, **kwargs):
+        """ Make a SoftLayer API call
 
         :param service: the name of the SoftLayer API service
         :param method: the method to call on the service
-        :param dict *args: arguments for the specified method
-        :param dict **kwargs: response-level arguments (limit, offset, etc.)
+        :param \*args: same optional arguments that ``Service.call`` takes
+        :param \*\*kwargs: same optional keyword arguments that
+                           ``Service.call`` takes
+
+        :param service: the name of the SoftLayer API service
+
+        Usage:
+            >>> client['Account'].getVirtualGuests(mask="id", limit=10)
+            [...]
 
         """
+        if kwargs.get('iter'):
+            return self.iter_call(service, method, *args, **kwargs)
+
+        if not service.startswith(self._prefix):
+            service = self._prefix + service
+
         objectid = kwargs.get('id')
         objectmask = kwargs.get('mask')
         objectfilter = kwargs.get('filter')
@@ -263,6 +262,59 @@ class Client(object):
                              http_headers=http_headers, timeout=self.timeout,
                              verbose=self.verbose)
 
+    __call__ = call
+
+    def iter_call(self, service, method,
+                  chunk=100, limit=None, offset=0, *args, **kwargs):
+        """ A generator that deals with paginating through results.
+
+        :param service: the name of the SoftLayer API service
+        :param method: the method to call on the service
+        :param integer chunk: result size for each API call
+        :param \*args: same optional arguments that ``Service.call`` takes
+        :param \*\*kwargs: same optional keyword arguments that
+                           ``Service.call`` takes
+
+        """
+        if chunk <= 0:
+            raise AttributeError("Chunk size should be greater than zero.")
+
+        if limit:
+            chunk = min(chunk, limit)
+
+        result_count = 0
+        kwargs['iter'] = False
+        while True:
+            if limit:
+                # We've reached the end of the results
+                if result_count >= limit:
+                    break
+
+                # Don't over-fetch past the given limit
+                if chunk + result_count > limit:
+                    chunk = limit - result_count
+            results = self.call(service, method,
+                                offset=offset, limit=chunk, *args, **kwargs)
+
+            # It looks like we ran out results
+            if not results:
+                break
+
+            # Apparently this method doesn't return a list.
+            # Why are you even iterating over this?
+            if not isinstance(results, list):
+                yield results
+                break
+
+            for item in results:
+                yield item
+                result_count += 1
+
+            offset += chunk
+
+            if len(results) < chunk:
+                break
+
     def __format_object_mask(self, objectmask, service):
         """ Format new and old style object masks into proper headers.
 
@@ -290,7 +342,6 @@ class Client(object):
 
     def __getattr__(self, name):
         """ Attempt a SoftLayer API call.
-        ..  deprecated:: 2.0.0
 
         Use this as a catch-all so users can call SoftLayer API methods
         directly against their client object. If the property or method
@@ -300,6 +351,8 @@ class Client(object):
 
         :param name: method name
 
+        ..  deprecated:: 2.0.0
+
         """
         if name in ["__name__", "__bases__"]:
             raise AttributeError("'Obj' object has no attribute '%s'" % name)
@@ -309,7 +362,7 @@ class Client(object):
                 raise SoftLayerError(
                     "Service is not set on Client instance.")
             kwargs['headers'] = self._headers
-            return self(self._service_name, name, *args, **kwargs)
+            return self.call(self._service_name, name, *args, **kwargs)
         return call_handler
 
     def __repr__(self):
@@ -324,8 +377,49 @@ class Service(object):
         self.client = client
         self.name = name
 
-    def __call__(self, name, *args, **kwargs):
-        return self.client(self.name, name, *args, **kwargs)
+    def call(self, name, *args, **kwargs):
+        """ Make a SoftLayer API call
+
+        :param method: the method to call on the service
+        :param \*args: (optional) arguments for the remote call
+        :param id: (optional) id for the resource
+        :param mask: (optional) object mask
+        :param dict filter: (optional) filter dict
+        :param dict headers: (optional) optional XML-RPC headers
+        :param dict raw_headers: (optional) HTTP transport headers
+        :param int limit: (optional) return at most this many results
+        :param int offset: (optional) offset results by this many
+        :param boolean iter: (optional) if True, returns a generator with the
+                             results
+
+        Usage:
+            >>> client['Account'].getVirtualGuests(mask="id", limit=10)
+            [...]
+
+        """
+        return self.client.call(self.name, name, *args, **kwargs)
+
+    __call__ = call
+
+    def iter_call(self, name, *args, **kwargs):
+        """ A generator that deals with paginating through results.
+
+        :param method: the method to call on the service
+        :param integer chunk: result size for each API call
+        :param \*args: same optional arguments that ``Service.call`` takes
+        :param \*\*kwargs: same optional keyword arguments that
+                           ``Service.call`` takes
+
+        Usage:
+            >>> gen = client['Account'].getVirtualGuests(iter=True)
+            >>> for virtual_guest in gen:
+            ...     virtual_guest['id']
+            ...
+            1234
+            4321
+
+        """
+        return self.client.iter_call(self.name, name, *args, **kwargs)
 
     def __getattr__(self, name):
         if name in ["__name__", "__bases__"]:
