@@ -1,3 +1,11 @@
+"""
+    SoftLayer.exceptions
+    ~~~~~~~~~~~~~~~~~~~~
+    Exceptions used throughout the library
+
+    :copyright: (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
+    :license: BSD, see LICENSE for more details.
+"""
 
 
 class SoftLayerError(StandardError):
