@@ -1,3 +1,11 @@
+"""
+    SoftLayer.firewall
+    ~~~~~~~~~~~~~~~~~~
+    Firewall Manager/helpers
+
+    :copyright: (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
+    :license: BSD, see LICENSE for more details.
+"""
 __all__ = ['FirewallManager']
 
 
